SystemOrganization addCategory: #'XML-Parser-HTML-Tests'!

TestCase subclass: #HTMLParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTML-Tests'!

!HTMLParserTest methodsFor: 'benchmarking' stamp: 'monty 2/5/2014 23:10'!
benchmark1
	"
	HTMLParserTest new benchmark1
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [HTMLHandler parse: XMLParserTest addressBookXML]] timeToRun! !

!HTMLParserTest methodsFor: 'benchmarking' stamp: 'monty 2/5/2014 23:07'!
benchmark2
	"
	HTMLParserTest new benchmark2
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [HTMLParser parse: XMLParserTest addressBookXML]] timeToRun! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 10/26/2014 14:21'!
testDecodingCharacters
	self assert:
		(HTMLParser parse: '&#60; &#x3C &#X3E &#62; &#0; &#-1 &#invalid; &#invalid &#')
			contentString = '< < > > &#0; &#-1 &#invalid; &#invalid &#'.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/10/2014 21:24'!
testHTMLEntities
	| entity character string |

	entity := 'nbsp'.
	character := HTMLParserTokenizer predefinedEntities at: entity.
	string :=
		((HTMLParser parse: '<p>&', entity, ';&', entity, '</p>')
			findElementNamed: 'p') contentString.
	self
		assert: string size = 2;
		assert: string first == character;
		assert: string second == character.
		
	self assert:
		((HTMLParser parse: '<body>&undeclared; &undeclared &;')
			findElementNamed: 'body') contentString = '&undeclared; &undeclared &;'.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/6/2014 03:56'!
testParsingAttributes
	| document element |

	document := HTMLParser parse: '<test one="quoted" two=''subQuoted'' three=unquoted four'.
	element := document findElementNamed: 'test'.
	self
		assert: (element attributeAt: 'one') = 'quoted';
		assert: (element attributeAt: 'two') = 'subQuoted';
		assert: (element attributeAt: 'three') = 'unquoted';
		assert: (element attributeAt: 'four') isEmpty.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:51'!
testParsingCDATA
	| document |

	document := HTMLParser parse: '<body><!![CDATA[ignored]]></body>'.
	self deny: (document findElementNamed: 'body') hasChildren.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/6/2014 18:46'!
testParsingForms
	| document |

	document := HTMLParser parse: '<form><input><form><input>'.

	self assert:
		(document allElements asArray collect: [:each | each name]) =
			#('html' 'body' 'form' 'input' 'form' 'input').

	document allElementsNamed: 'form' do: [:each |
		self
			assert: each elements size = 1;
			assert: (each firstElement isNamed: 'input')].
	document allElementsNamed: 'input' do: [:each |
		self deny: each hasChildren].! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/15/2014 01:20'!
testParsingImplicitStartTag
	| paragraphs |

	paragraphs := (HTMLParser parse: '</p></p>') allElementsNamed: 'p'.
	self
		assert: paragraphs size = 2;
		deny: (paragraphs anySatisfy: [:each | each hasChildren]).! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/4/2014 02:50'!
testParsingInvalidPCData
	| document |

	#('< one & two >' '& three < four >') do: [:each |
		document := HTMLParser parse: each.
		self
			assert: document allNodes size = 4;
			assert: (document findElementNamed: 'body') contentString = each].! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 1/16/2014 05:09'!
testParsingNonHTMLSections
	| document |

	document := HTMLParser parse:
		'<head><Title><b>"title</b></TITLE><SCRIPT>',
		'1 < 2; 3 > 2; document.write("</Script>");</Script>
		<Style>//"</STYLE><HEAD>'.
	self
		assert: (document findElementNamed: 'title') contentString = '<b>"title</b>';
		assert: (document findElementNamed: 'script') contentString =
			'1 < 2; 3 > 2; document.write("</Script>");';
		assert: (document findElementNamed: 'style') contentString = '//"'.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/5/2014 23:45'!
testParsingTables
	| document tables trs tds |

	document := HTMLParser parse: '<table><tr><td>one<td>two<tr><td>three<table><tr><td>four'.
	self assert:
		(document allElements asArray collect: [:each | each name]) =
			#('html' 'body' 'table' 'tr' 'td' 'td' 'tr' 'td' 'table' 'tr' 'td').

	tables := document allElementsNamed: 'table'.
	self
		assert: tables first elements size = 2;
		assert: tables second elements size = 1.
	tables do: [:each | self assert: (each elements allSatisfy: [:element | element isNamed: 'tr'])].

	trs := document allElementsNamed: 'tr'.
	self
		assert: trs first elements size = 2;
		assert: trs second elements size = 1;
		assert: trs third elements size = 1.
	trs do: [:each | self assert: (each elements allSatisfy: [:element | element isNamed: 'td'])].

	tds := document allElementsNamed: 'td'.
	self
		assert: tds first contentString = 'one';
		assert: tds second contentString = 'two';
		assert: tds third contentString = 'three four';
		assert: (tds third nodes second isNamed: 'table').! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 12/11/2013 22:00'!
testParsingText
	| document |
	
	document := HTMLParser parse: 'text'.
	self
		assert: document allNodes size = 4;
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body');
		assert: (document findElementNamed: 'body') contentString = 'text'.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 12/11/2013 22:03'!
testParsingVoidElements
	| document |

	document := HTMLParser parse: '<body>one<img>two</img><body>three'.
	self
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body' 'img');
		assert: (document findElementNamed: 'body') contentString = 'one twothree';
		deny: (document findElementNamed: 'img') hasChildren.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/3/2014 22:53'!
testParsingXML
	self
		shouldnt: [
			HTMLHandler
				parse: XMLParserTest addressBookXML;
				parse: XMLParserTest addressBookXMLWithDTD;
				parse: XMLDOMParserTest addressBookNamespacedXML;
				parse: XMLDOMParserTest addressBookNamespacePrefixedXML]
		raise: XMLParserException! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/11/2014 00:51'!
testProcessingInstructions
	| document |

	document := HTMLParser parse: '<body><?ignored data?></body>'.
	self deny: (document findElementNamed: 'body') hasChildren.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 12/13/2013 20:11'!
testReopeningHTML
	| document |

	document := HTMLParser parse: '<html><B></body></html><I>'.
	self
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body' 'b' 'i');
		assert: document root nodes size = 1;
		assert: (document root firstElement elementsCollect: [:each | each name]) asArray = #('b' 'i').

	document := HTMLParser parse: '<html><B>bold</body></html>normal'.
	self
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body' 'b');
		assert: (document findElementNamed: 'body') contentString = 'bold normal'.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 1/16/2014 22:21'!
testReopeningHeadSection
	| document |

	document := HTMLParser parse: '<head><p>text<Title>reopened'.
	self
		assert: (document root elementsCollect: [:each | each name]) asArray = #('head' 'body');
		assert: (document findElementNamed: 'head') elements size = 1;
		assert: ((document findElementNamed: 'head') elementAt: 'title') contentString = 'reopened';
		assert: (document findElementNamed: 'body') elements size = 1;
		assert: ((document findElementNamed: 'body') elementAt: 'p') contentString = 'text'.! !

!HTMLParserTest methodsFor: 'tests' stamp: 'monty 2/9/2014 01:46'!
testXMLDeclaration
	| document |

	document := HTMLParser parse: '<?xml ?>'.
	self
		assert: document version = 1.0;
		assert: document encoding = 'UTF-8';
		deny: document isStandalone.

	document := HTMLParser parse: '<?xml version="onePointZero" encoding="UTF-16" standalone="yes"?>'.
	self
		assert: document version = 1.0;
		assert: document encoding = 'UTF-16';
		deny: document isStandalone.! !
